package com.flow.delegate;

import com.flow.entity.FlowDefine;
import com.flow.entity.Notify;
import com.flow.entity.User;
import com.flow.entity.node.NotifyNode;
import com.flow.enums.NotifyEnum;
import com.flow.enums.NotifyTypeEnum;
import com.flow.service.FlowDefineService;
import com.flow.service.NotifyService;
import com.flow.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.flowable.common.engine.impl.el.ExpressionManager;
import org.flowable.engine.delegate.DelegateExecution;
import org.flowable.engine.delegate.JavaDelegate;
import org.flowable.engine.impl.cfg.ProcessEngineConfigurationImpl;
import org.flowable.engine.impl.persistence.entity.ExecutionEntityImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;


@Slf4j
@Component
public class NotifyDelegate implements JavaDelegate {
    @Autowired
    private FlowDefineService flowDefineService;
    @Autowired
    private JavaMailSender mailSender;
    @Value("${spring.mail.username}")
    private String from;
    @Autowired
    private NotifyService notifyService;
    @Autowired
    private UserService userService;
    @Autowired
    private ProcessEngineConfigurationImpl processEngineConfiguration;

    @Override
    public void execute(DelegateExecution execution) {
        ExecutionEntityImpl entity = (ExecutionEntityImpl) execution;
        String activityId = execution.getCurrentActivityId();
        FlowDefine define = flowDefineService.getDefine(entity.getProcessDefinitionId());
        NotifyNode notifyNode = define.getNode(execution.getCurrentActivityId(), NotifyNode.class);
        List<NotifyTypeEnum> typeEnums = notifyNode.getTypes();
        ExpressionManager expressionManager = processEngineConfiguration.getExpressionManager();
        String content = (String) expressionManager.createExpression(notifyNode.getContent()).getValue(execution);
        String subject = (String) expressionManager.createExpression(notifyNode.getSubject()).getValue(execution);
        // 通知
        String assignee = execution.getVariableLocal(String.format("%sItem", activityId), String.class);
        User user = userService.getByUsername(assignee);
        for (NotifyTypeEnum typeEnum : typeEnums) {
            switch (typeEnum) {
                case SITE:
                    // 站内通知
                    Notify notify = new Notify();
                    notify.setSender("admin");
                    notify.setType(NotifyEnum.SYSTEM);
                    notify.setReceivingTime(LocalDateTime.now());
                    notify.setReceiver(assignee);
                    notify.setSubject(subject);
                    notify.setContent(content);
                    notifyService.notify(notify);
                    break;
                case EMAIL:
                    // 邮件通知
                    SimpleMailMessage message = new SimpleMailMessage();
                    message.setFrom(this.from);
                    message.setTo(user.getEmail());
                    message.setSubject(subject);
                    message.setText(content);
                    mailSender.send(message);
                    break;
                default:
                    break;
            }
        }
    }
}
